﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Windows;
using System.Windows.Data;

namespace SampleApp
{

    public class AxisLabelsVisibilityConverter : IValueConverter
    {
        public AxisLabelsVisibilityConverter()
        {
            IsInitialized = false;
            IsUpadting = true;
            LabelsCounter = new Dictionary<double, int>();
        }
        public bool IsInitialized { get; set; }
        public bool IsUpadting { get; set; }

        protected Dictionary<double, int> LabelsCounter;
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value is double && IsInitialized && IsUpadting)
            {
                var doubleValue = (double) value;
                var floorValue = System.Math.Floor(doubleValue);

                if ((doubleValue - floorValue) > 0)
                {
                    System.Diagnostics.Debug.WriteLine("skipping axis label " + doubleValue + " diff " + (doubleValue - floorValue));
                    return Visibility.Collapsed;
                }

                if (LabelsCounter.ContainsKey(floorValue))
                {
                    LabelsCounter[floorValue]++;

                    if (LabelsCounter[floorValue] > 2)
                    {
                        System.Diagnostics.Debug.WriteLine("skipping axis label " + floorValue);
                        return Visibility.Collapsed;
                    }
                }
                else
                {
                    System.Diagnostics.Debug.WriteLine("added axis label " + floorValue);
                    LabelsCounter.Add(floorValue, 1);
                }
            }
            return Visibility.Visible;
        }
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        public void ResetLabelsCounter()
        {
            IsInitialized = true;
            LabelsCounter.Clear();
        }
    }
    public class AxisLabelsFloorConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value is double)
            {
                var label = System.Math.Floor((double)value).ToString();
                //label += "(" + value + ")";
                return label;
            }
            return value;
        }
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}