﻿using System.ComponentModel;

namespace SampleApp
{
    /// <summary>
    /// Represents an observable model that implements INotifyPropertyChanged interface
    /// </summary>
    public abstract class ObservableModel : INotifyPropertyChanged
    {

        #region INotifyPropertyChanged

        /// <summary>
        /// Occurs when a property value was changed
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        protected bool HasPropertyChangedHandler()
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            return handler != null;
        }
        protected void OnPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
                handler(sender, e);
        }
        protected void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            OnPropertyChanged(this, e);
        }
        protected void OnPropertyChanged(object sender, string propertyName)
        {
            OnPropertyChanged(sender, new PropertyChangedEventArgs(propertyName));
        }
        /// <summary>
        /// Raises the PropertyChanged event for provided property name
        /// </summary>
        /// <param name="propertyName"></param>
        public void OnPropertyChanged(string propertyName)
        {
            OnPropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        
        protected delegate void PropertyChangedDelegate(object sender, string propertyName);

#if SILVERLIGHT
        public void OnAsyncPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler == null) return;

            Deployment.Current.Dispatcher.BeginInvoke(new PropertyChangedDelegate(OnPropertyChanged), this, propertyName);
            //Deployment.Current.Dispatcher.BeginInvoke(() =>
            //{
            //    OnPropertyChanged(this, propertyName);
            //});
        }
#endif
        #endregion

    }

}