﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;

namespace SampleApp
{
    /// <summary>
    /// Represents sample data
    /// </summary>
    public class SampleData : DataList
    {
        public SampleData()
        {
            var rand = new Random();

            for (int i = 0; i < 400; i++)
            {
                var x = rand.Next(0, 100);
                var y = rand.Next(0, 100);
                var v = rand.Next(10, 50);
                this.Add(new DataItem { X = x, Y = y, Value = v, Index = i });
            }
        }
    }

    /// <summary>
    /// Represents a list of data item
    /// </summary>
    public class DataList : List<DataItem>
    { }

    /// <summary>
    /// Represents a single data item
    /// </summary>
    public class DataItem  
    {

        public new string ToString()
        {
            return Label + ", " + Value + ", " + X + ", " + Y;
        }
        public string Label { get; set; }
        public int Index { get; set; }
    
        public double Value { get; set; }
        public double X { get; set; }
        public double Y { get; set; }
    }

    /// <summary>
    /// Represents data view model 
    /// </summary>
    public class DataViewModel : ObservableModel
    {
        public DataViewModel()
        {
            //NOTE you can provide or load your own data here
            var sampleData = new SampleData();

            Items = new List<MarkerModel>();
            foreach (var item in sampleData)
            {
                Items.Add(new MarkerModel { Data = item });
            }
        }

        public List<MarkerModel> Items { get; set; }
      
    }
    /// <summary>
    /// Represents view model for a Marker and provides access to DataItem
    /// </summary>
    public class MarkerModel : Control
    {
        public MarkerModel()
        {
            Data = new DataItem();
            //default to Collapsed value
            Visibility = Visibility.Collapsed;
        }
        public DataItem Data { get; set; }
    }

    // Alternative implemenation of MarkerModel

    ///// <summary>
    ///// Represents view model for a Marker and provides access to DataItem
    ///// </summary>
    //public class MarkerModel : ObservableModel
    //{
    //    public MarkerModel()
    //    {
    //        Data = new DataItem(); 
    //        HighlightVisibility = Visibility.Collapsed;
    //    }
    //    public DataItem Data { get; set; }

    //    //NOTE: you can add more properties with OnPropertyChanged notififaction and
    //    // bind them to in Marker data template, e.g. "{Binding Path=Item.PROPERTY_NAME}"
        
    //    private Visibility _highlightVisibility;
    //    /// <summary>
    //    /// Gets or sets IsHighlighted property 
    //    /// </summary>
    //    public Visibility HighlightVisibility
    //    {
    //        get { return _highlightVisibility; }
    //        set { _highlightVisibility = value; OnPropertyChanged("HighlightVisibility"); }
    //    }

    //}


}