﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace SampleApp
{
    /// <summary>
    /// Interaction logic for ChartAnnotationMarkers.xaml
    /// </summary>
    public partial class ChartAnnotationMarkers : UserControl
    {
        public ChartAnnotationMarkers()
        {
            InitializeComponent();
            this.Unloaded += ChartAnnotationMarkers_Unloaded;
        }

        void ChartAnnotationMarkers_Unloaded(object sender, RoutedEventArgs e)
        {
            // when app is closing:
            //TODO add logic to save DataContext of the XamDataChart by 
            //TODO saving value from Data property for each Items of the DataViewModel
        }

       
        private MarkerControl GetClosestMarker(ScatterSeries series, Point location)
        {
            //NOTE: this values should match dimensions of visible element in Marker data template
            //with optional small margin, e.g. 2.0
            var width = 12.0;
            var height = 12.0;
            var widthHalf = width / 2;
            var heightHalf = height / 2;

            MarkerControl marker = null;
            if (series != null)
            {
                var items = series.ItemsSource as List<MarkerControl>;
                if (items != null)
                {
                    var index = -1;
                    // find a markers that was hovered over
                    foreach (var item in items)
                    {
                        index++;
                        // scale item location to series' coordinates 
                        var x = series.XAxis.ScaleValue(item.Data.X);
                        var y = series.YAxis.ScaleValue(item.Data.Y);
                        var bounds = new Rect(x - widthHalf, y - heightHalf, width, height);

                        if (bounds.Contains(location))
                        {
                            //LastHighligtedMarkerIndex = index;
                            marker = item;
                            item.ZOrder = 1000;
                        }
                        else
                        {
                            item.ZOrder = 1;
                        }
                    }
                }
            }
            LastHighligtedMarkerIndex = -1;
            //LastAnnotatedMarkerIndex = -1;
            return marker;
        }
  
        #region Chart Highliting
        protected int LastHighligtedMarkerIndex = -1;
        private void ClearMarkerHighligting(ScatterSeries series)
        {
            if (LastHighligtedMarkerIndex < 0) return;
            if (series != null)
            {
                var items = series.ItemsSource as List<MarkerControl>;
                if (items != null)
                {
                    if (items.Count > 0 && items.Count > LastHighligtedMarkerIndex)
                    {
                        items[LastHighligtedMarkerIndex].HideHighlight();
                    }
                }
            }
        }
        private void DataChart_OnSeriesMouseMove(object sender, ChartMouseEventArgs e)
        {
            var hoverSeries = e.Series as ScatterSeries;
            var dataSeries = DataChart.Series.OfType<ScatterSeries>().First();
            if (hoverSeries != null && hoverSeries.Name != dataSeries.Name) return;

            var seriesLocation = e.GetPosition(dataSeries);
            ClearMarkerHighligting(dataSeries);
            var item = GetClosestMarker(dataSeries, seriesLocation);
            if (item != null)
            {
                LastHighligtedMarkerIndex = item.Data.Index;
                item.HighlightControl.Visibility = Visibility.Visible;
            }
        }
        private void DataChart_OnSeriesMouseLeave(object sender, ChartMouseEventArgs e)
        {
            var hoverSeries = e.Series as ScatterSeries;
            var dataSeries = DataChart.Series.OfType<ScatterSeries>().First();
            if (hoverSeries != null && hoverSeries.Name != dataSeries.Name) return;
            //var series = e.Series as ScatterSeries;
            ClearMarkerHighligting(dataSeries);
            //ClearMarkerAnnotaion(series);
        }

        #endregion
        #region Chart Annotation
        protected int LastAnnotatedMarkerIndex = -1;
        private void ClearMarkerAnnotaion(ScatterSeries series)
        {
            if (LastAnnotatedMarkerIndex < 0) return;
            if (series != null)
            {
                var items = series.ItemsSource as List<MarkerControl>;
                if (items != null)
                {
                    if (items.Count > 0 && items.Count > LastAnnotatedMarkerIndex)
                    {
                        items[LastAnnotatedMarkerIndex].HideAnnotaion();
                    }
                }
            }
            //var annotationSeries = DataChart.Series.OfType<ScatterSeries>().Last();
            //annotationSeries.ItemsSource = null; // new List<MarkerControl>() { };
        
        }
        private void DataChart_OnSeriesMouseClick(object sender, DataChartMouseButtonEventArgs e)
        {
            var dataSeries = DataChart.Series.OfType<ScatterSeries>().First();
            
            //var series = e.Series as ScatterSeries;
            var seriesLocation = e.GetPosition(dataSeries);
            
            ClearMarkerAnnotaion(dataSeries);
            ClearMarkerHighligting(dataSeries);
            var item = GetClosestMarker(dataSeries, seriesLocation);
            if (item != null)
            {
                item.ZOrder = 1000;
                item.ShowAnnotaion();
                var annotationSeries = DataChart.Series.OfType<ScatterSeries>().Last();
                annotationSeries.ItemsSource = new List<MarkerControl>() {item};

                LastHighligtedMarkerIndex = item.Data.Index;
                LastAnnotatedMarkerIndex = item.Data.Index;
                //item.ShowHighlight();
                //item.ShowAnnotaion();
            }
            e.Handled = true;
        }

        #endregion

        private void DataChart_OnMouseClick(object sender, MouseButtonEventArgs e)
        {
            var dataSeries = DataChart.Series.OfType<ScatterSeries>().First();
            ClearMarkerHighligting(dataSeries);
            var annotationSeries = DataChart.Series.OfType<ScatterSeries>().Last();
            annotationSeries.ItemsSource = null; // new List<MarkerControl>() { };
            e.Handled = true;
        }
        private void TextBoxBase_OnTextChanged(object sender, TextChangedEventArgs e)
        {
            var control = sender as TextBox;
            if (control != null)
            {
                var dc = control.DataContext as ScatterDataItem;
                if (dc != null)
                    dc.AnnotaionText = control.Text;
            }
        }

        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            var control = sender as Button;
            if (control != null)
            {
                var dc = control.DataContext as ScatterDataItem;
                if (dc != null)
                    dc.AnnotaionText = string.Empty;
            }

        }
    }

    /// <summary>
    /// Represents data view model 
    /// </summary>
    public class DataViewModel : ObservableModel
    {
        public DataViewModel()
        {
            //NOTE you can provide or load your own data here
            var sampleData = new ScatterSampleData();

            Items = new List<MarkerControl>();
            for (int i = 0; i < sampleData.Count() - 1; i++)
            {
                sampleData[i].Index = i;
                Items.Add(new MarkerControl { Data = sampleData[i] });
            }
        }

        public List<MarkerControl> Items { get; set; }

    }
}
