﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Infragistics.Controls.Charts;

namespace SampleApp
{
    public partial class ChartInfoStrips : UserControl
    {
        public ChartInfoStrips()
        {
            InitializeComponent();

            this.Loaded += ChartInfoStrips_Loaded;
        }

        void ChartInfoStrips_Loaded(object sender, System.Windows.RoutedEventArgs e)
        {

            var dateStart = new DateTime(2010, 1, 1);
            var dateEnd = new DateTime(2010, 4, 1);

            var infoRanges = new List<InfoStripRange>();
            infoRanges.Add(new InfoStripRange { Label = dateStart.Year + " Q1", Start = dateStart, End = dateEnd });
            dateStart = dateStart.AddMonths(6);
            dateEnd = dateEnd.AddMonths(6);
            infoRanges.Add(new InfoStripRange { Label = dateStart.Year + " Q3", Start = dateStart, End = dateEnd });
            dateStart = dateStart.AddMonths(6);
            dateEnd = dateEnd.AddMonths(6);
            infoRanges.Add(new InfoStripRange { Label = dateStart.Year + " Q1", Start = dateStart, End = dateEnd });
            dateStart = dateStart.AddMonths(6);
            dateEnd = dateEnd.AddMonths(6);
            infoRanges.Add(new InfoStripRange { Label = dateStart.Year + " Q3", Start = dateStart, End = dateEnd });
            dateStart = dateStart.AddMonths(6);
            dateEnd = dateEnd.AddMonths(6);
            infoRanges.Add(new InfoStripRange { Label = dateStart.Year + " Q1", Start = dateStart, End = dateEnd });
            dateStart = dateStart.AddMonths(6);
            dateEnd = dateEnd.AddMonths(6);
            infoRanges.Add(new InfoStripRange { Label = dateStart.Year + " Q3", Start = dateStart, End = dateEnd });

            //var chartStripBehaviours = new ChartStripBehaviors();

            var chartInfoBehaviour = new ChartInfoStripsBehavior();
            var verticalInfoStripTemplate = this.Resources["VerticalInfoStripTemplate"] as DataTemplate;
            var i = 0;
            foreach (var infoRange in infoRanges)
            {
                var infoStrip = new InfoStrip();
                infoStrip.StartDate = infoRange.Start;
                infoStrip.EndDate = infoRange.End;
                infoStrip.Label = infoRange.Label;
                //infoStrip.Label = "Q" + ( ++i);
                infoStrip.UseDates = true;
                infoStrip.StripTemplate = verticalInfoStripTemplate;
                infoStrip.Fill = new SolidColorBrush(Colors.Gray) { Opacity = 0.5};
                chartInfoBehaviour.InfoStrips.Add(infoStrip);
            }
            
            ChartStripBehaviors.SetChartInfoStrips(DataChart2, chartInfoBehaviour);
        }
    }

    public class InfoStripRange
    {
        public DateTime Start { get; set; }
        public DateTime End { get; set; }
        public string Label { get; set; }
    }

    public class StripDataCollection : ObservableCollection<StripDataPoint>
    {
        private static readonly Random DataGenerator = new Random();

        public StripDataCollection()
        {
            this.Generate();
        }

        private void Generate()
        {
            this.Clear();
            double curr = 100;
            var date = new DateTime(2009, 6, 01, 12, 00, 00); //DateTime.Now;

            for (int i = 0; i < 52; i++)
            {
                if (DataGenerator.NextDouble() > .5)
                {
                    curr += DataGenerator.NextDouble();
                }
                else
                {
                    curr -= DataGenerator.NextDouble();
                }
                this.Add(
                    new StripDataPoint
                    {
                        Date = date,
                        Index = i,
                        Label = (i + 1).ToString(),
                        Value = curr
                    });
                date = date.AddMonths(1);
            }

        }
    }
    public class StripDataPoint
    {
        public string Label { get; set; }
        public double Value { get; set; }
        public int Index { get; set; }
        public DateTime Date { get; set; }
    }
}
