﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace StackedChart
{
    /// <summary>
    /// Interaction logic for StackColumnSeriesWithMarkers.xaml
    /// </summary>
    public partial class StackColumnSeriesWithMarkers : UserControl
    {
        public StackColumnSeriesWithMarkers()
        {
            InitializeComponent();

            this.Loaded += Contol_Loaded;
        }
        public DataTable DataTable { get; set; }

        private void LoadDataTable()
        {
            DataTable = new DataTable();
            DataTable.Columns.Add("ID", typeof(string));
            DataTable.Columns.Add("Type", typeof(string));
            DataTable.Columns.Add("Title", typeof(string));
            DataTable.Columns.Add("Description", typeof(string));
            DataTable.Columns.Add("Priority", typeof(string));
            DataTable.Columns.Add("Category", typeof(string));
            DataTable.Columns.Add("Source", typeof(string));
            DataTable.Columns.Add("CreatedOn", typeof(DateTime));

            var row = DataTable.NewRow();

            row["ID"] = "AR142";
            row["Type"] = "Activity";
            row["Title"] = "Any activity";
            row["Description"] = "Bla bla bla....";
            row["Priority"] = "Low";
            row["Category"] = "";
            row["Source"] = "";
            row["CreatedOn"] = DateTime.Parse("2013-05-30 1:30:00 PM");
            DataTable.Rows.Add(row);

            row = DataTable.NewRow();
            row["ID"] = "IR102";
            row["Type"] = "Incident";
            row["Title"] = "Any incident";
            row["Description"] = "Bla bla bla.......";
            row["Priority"] = "High";
            row["Category"] = "Network";
            row["Source"] = "Portal";
            row["CreatedOn"] = DateTime.Parse("2013-05-28 1:00:00 PM");
            DataTable.Rows.Add(row);

            row = DataTable.NewRow();
            row["ID"] = "IR11112";
            row["Type"] = "Incident";
            row["Title"] = "Any incidentadsa";
            row["Description"] = "Bla bla bla.......";
            row["Priority"] = "Low";
            row["Category"] = "Hardware";
            row["Source"] = "Portal";
            row["CreatedOn"] = DateTime.Parse("2013-05-28 1:00:00 PM");
            DataTable.Rows.Add(row);

            row = DataTable.NewRow();
            row["ID"] = "IR152";
            row["Type"] = "Incident";
            row["Title"] = "Any incident 22";
            row["Description"] = "Bla bla";
            row["Priority"] = "Low";
            row["Category"] = "Network";
            row["Source"] = "Phone";
            row["CreatedOn"] = DateTime.Parse("2013-05-29 3:45:00 AM");
            DataTable.Rows.Add(row);

            row = DataTable.NewRow();
            row["ID"] = "PR104";
            row["Type"] = "Problem";
            row["Title"] = "Any problem";
            row["Description"] = "Bla bla bla...";
            row["Priority"] = "Medium";
            row["Category"] = "Software";
            row["Source"] = "Portal";
            row["CreatedOn"] = DateTime.Parse("2013-04-28 7:30:00 AM");
            DataTable.Rows.Add(row);

            row = DataTable.NewRow();
            row["ID"] = "PR172";
            row["Type"] = "Problem";
            row["Title"] = "Any problem";
            row["Description"] = "Bla bla bla";
            row["Priority"] = "High";
            row["Category"] = "";
            row["Source"] = "Manual";
            row["CreatedOn"] = DateTime.Parse("2013-04-28 12:05:00 PM");
            DataTable.Rows.Add(row);

            row = DataTable.NewRow();
            row["ID"] = "RRR104";
            row["Type"] = "Release";
            row["Title"] = "Any release";
            row["Description"] = "";
            row["Priority"] = "High";
            row["Category"] = "";
            row["Source"] = "Automatic";
            row["CreatedOn"] = DateTime.Parse("2011-04-28 5:30:00 AM");
            DataTable.Rows.Add(row);


        }
        private void PlotDataTable(string groupName, string categoryName)
        {
            #region Query

            //This one is to make a multi column group by statement
            var query = from table in DataTable.AsEnumerable()
                        group table by new
                        {
                            Type = table.Field<string>(groupName),
                            Category = table.Field<string>(categoryName)
                        };

            //This one is to calculate the counts of the selected groups of the second column
            var summary = from cc in query
                          select new
                          {
                              Type = cc.Key.Type,
                              Category = cc.Key.Category,
                              Count = cc.Count()
                          };
            //var summaryList = summary.ToList();

            //This one is to re-construct the summary in the form of [First Column] ---> [List of Groups of the Second Column]
            var groupedSummary = from grp in summary
                                 group grp by grp.Type into groupedValues
                                 select new
                                 {
                                     Name = groupedValues.Key,
                                     Count = groupedValues.Count(),
                                     CategoryValues = groupedValues
                                 };
            //var groupList = groupedSummary.ToList();

            #endregion
            /*
             * Here I want to draw the stacked chart based on the group summary meaning that 
             * for each type in the grouped summary there are groups of categories.
             * In the real scenario this is a very dynamic graph meaning that the user can select 
             * any 2 column names from the data table and he/she should see the stacked graph
             * which means that the names of the two columns used in "query" can be any column names. 
             * I just selected "Type" and "Category" here in the example. Also take into 
             * consideration that some columns will not have values for the selected group. 
             */

            /*
             * I really need your help to be able to do the following in the "XamDataChart"
             * 1- Draw the stacked graph
             * 2- Have the value of each stack either written inside it or appears when I hover on it with the mouse
             * 3- Have the a marker with the value of the column on the top of each column
             * 4- Have a legend where all the stacks appear in it
             */

            var dataSource = new GroupDataList();

            // clear stack segments  
            var stackColumnSeries = xDChart.Series.OfType<StackedColumnSeries>().First();
            stackColumnSeries.Series.Clear();
            // remove point series (marker series) in case they exist in the chart
            for (int i = 0; i < xDChart.Series.Count; i++)
            {
                if (xDChart.Series.Count > 1 && xDChart.Series[1] is PointSeries)
                    xDChart.Series.RemoveAt(1);
            }

            var categoryNames = new List<string>();
            // create data items and stack segments based on values from the query 
            foreach (var group in groupedSummary)
            {
                var issueGroup = new GroupDataItem();
                if (group.Name == string.Empty)
                {
                    issueGroup.Name = "Unknown"; // force some group name
                }
                else
                {
                    issueGroup.Name = group.Name;
                }
                //issueGroup.GroupCount = group.Count;

                var categoryItems = group.CategoryValues.ToList();
                for (int i = 0; i < categoryItems.Count(); i++)
                {
                    /*
                     * this for loop is responsible for filling 
                     * the "StackedFragmentSeries" and add it to the "StackedColumnSeries" 
                     * which will be added after the loop finishes to the "XamDataChart" 
                     * which in our case is called "xDChart"
                     */

                    var item = categoryItems[i];
                    var category = item.Category;
                    if (category == string.Empty)
                    {
                        category = "Unknown"; // force some category name
                    }

                    // collect all unique names of categories and create stack segments
                    if (!categoryNames.Contains(category))
                    {
                        categoryNames.Add(category);
                        var title = category;

                        // map category value based on its name
                        var stackSegement = new StackedFragmentSeries();
                        stackSegement.ValueMemberPath = "CategoryValues[" + category + "]";
                        stackSegement.Title = title;
                        stackColumnSeries.Series.Add(stackSegement);

                    }
                    // store values of categories based on their name
                    if (!issueGroup.CategoryValues.ContainsKey(category))
                    {
                        issueGroup.CategoryValues.Add(category, item.Count);
                    }
                    else
                    {
                        // sum up values when case categories are duplicated
                        //issueGroup.Dictionary[categoryName] += item.Count;
                    }
                }
                dataSource.Add(issueGroup);
            }

            // force default values for non-existing categories in data source
            // this is required because data binding is using Dictionary and all categories must exist
            foreach (var item in dataSource)
            {
                item.CategoryOrderList = categoryNames;
                item.UpdateStacks();
                foreach (var category in categoryNames)
                {
                    if (!item.CategoryValues.ContainsKey(category))
                    {
                        item.CategoryValues.Add(category, 0);
                    }
                }
            }
            // create markers inside stack segments using Point Series with custom marker template
            var markerInsideColumnTemplate = this.RootGrid.Resources["MarkerInsideColumnTemplate"] as DataTemplate;
            foreach (var category in categoryNames)
            {
                var segmentMarkerItems = new List<MarkerItem>();
                foreach (var item in dataSource)
                {
                    if (item.StackPositions.ContainsKey(category) &&
                        item.StackValues.ContainsKey(category))
                        segmentMarkerItems.Add(new MarkerItem { Name = item.Name, Position = item.StackPositions[category], Value = item.StackValues[category] });
                    else
                        segmentMarkerItems.Add(new MarkerItem { Name = item.Name, Position = 0, Value = 0 });
                }

                var markerStackSeries = new PointSeries();
                markerStackSeries.XAxis = stackColumnSeries.XAxis;
                markerStackSeries.YAxis = stackColumnSeries.YAxis;
                markerStackSeries.MarkerTemplate = markerInsideColumnTemplate;
                markerStackSeries.ValueMemberPath = "Position";
                markerStackSeries.ItemsSource = segmentMarkerItems;
                xDChart.Series.Add(markerStackSeries);
            }

            // create markers on top of stack column series
            var markerOnTopColumnTemplate = this.RootGrid.Resources["MarkerOnTopColumnTemplate"] as DataTemplate;
            var markerColumnSeries = new PointSeries();
            markerColumnSeries.XAxis = stackColumnSeries.XAxis;
            markerColumnSeries.YAxis = stackColumnSeries.YAxis;
            markerColumnSeries.MarkerTemplate = markerOnTopColumnTemplate;
            markerColumnSeries.ValueMemberPath = "StackValue";
            xDChart.Series.Add(markerColumnSeries);

            // bind series and axes to data source
            markerColumnSeries.ItemsSource = dataSource;
            stackColumnSeries.ItemsSource = dataSource;

            var xAxis = xDChart.Axes.OfType<CategoryXAxis>().First();
            xAxis.ItemsSource = dataSource;

        }
        private void Contol_Loaded(object sender, RoutedEventArgs e)
        {
            LoadDataTable();

            //TODO: call this method each time your query is changed 
            //PlotDataTable("Source", "Category");
            //PlotDataTable("Priority", "Category");
            PlotDataTable("Type", "Category");
        }
    
    }

    public class MarkerItem
    {
        public string Name { get; set; }
        public double Position { get; set; }
        public double Value { get; set; }
    }

    public class GroupDataList : List<GroupDataItem>
    {

    }
    public class GroupDataItem
    {
        public string Name { get; set; }
        public int GroupCount { get; set; }

        public Dictionary<string, double> CategoryValues { get; set; }
        public Dictionary<string, double> StackPositions { get; private set; }
        public Dictionary<string, double> StackValues { get; private set; }

        public List<string> CategoryOrderList { get; set; }
        public GroupDataItem()
        {
            StackValues = new Dictionary<string, double>();
            StackPositions = new Dictionary<string, double>();
            CategoryValues = new Dictionary<string, double>();
            CategoryOrderList = new List<string>();
        }

        public double StackValue
        {
            get
            {
                var value = 0.0;
                foreach (var category in CategoryValues)
                {
                    value += category.Value;
                }
                return value;
            }
        }
        public void UpdateStacks()
        {
            StackPositions.Clear();
            StackValues.Clear();
            foreach (var category in CategoryValues)
            {
                var stackPosition = 0.0;
                var stackValue = 0.0;
                foreach (var name in CategoryOrderList)
                {
                    if (category.Key == name)
                        break;

                    if (CategoryValues.ContainsKey(name))
                        stackPosition += CategoryValues[name];
                }
                // middle of stack segment
                stackPosition += (category.Value / 2);
                stackValue += category.Value;

                StackPositions.Add(category.Key, stackPosition);
                StackValues.Add(category.Key, stackValue);
            }
        }

    }
   

}
